"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __exportStar = (this && this.__exportStar) || function(m, exports) {
    for (var p in m) if (p !== "default" && !Object.prototype.hasOwnProperty.call(exports, p)) __createBinding(exports, m, p);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fastify_plugin_1 = __importDefault(require("fastify-plugin"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const glob_promise_1 = __importDefault(require("glob-promise"));
const http_class_1 = require("http-class");
const ERROR_LABEL = 'fastify-autosecurity';
exports.default = fastify_plugin_1.default((fastify, options, next) => __awaiter(void 0, void 0, void 0, function* () {
    const { dir } = Object.assign(Object.assign({}, options), { dir: options.dir || './security' });
    const overrides = options.overrides || {};
    let dirPath;
    if (path_1.default.isAbsolute(dir)) {
        dirPath = dir;
    }
    else if (path_1.default.isAbsolute(process.argv[1])) {
        dirPath = path_1.default.join(process.argv[1], dir);
    }
    else {
        dirPath = path_1.default.join(process.cwd(), process.argv[1], dir);
    }
    if (!fs_1.default.existsSync(dirPath)) {
        return next(new Error(`${ERROR_LABEL} dir ${dirPath} does not exists`));
    }
    if (!fs_1.default.statSync(dirPath).isDirectory()) {
        return next(new Error(`${ERROR_LABEL} dir ${dirPath} must be a directory`));
    }
    const securities = yield glob_promise_1.default(`${dirPath}/*.[t|j]s`);
    const securityModules = {};
    for (const security of securities) {
        const securityName = security
            .replace(dirPath + '/', '')
            .replace('.js', '')
            .replace('.ts', '');
        securityModules[securityName] =
            securityName in overrides
                ? overrides[securityName](fastify)
                : loadModule(securityName, security)(fastify);
    }
    fastify.addHook('onRoute', (route) => {
        var _a, _b, _c, _d;
        if ((_a = route === null || route === void 0 ? void 0 : route.schema) === null || _a === void 0 ? void 0 : _a.security) {
            for (const security of route.schema.security) {
                for (const [securityName, securityScopes] of Object.entries(security)) {
                    if (securityName in securityModules === false) {
                        throw new Error(`[ERROR]: security "${securityName}" is not defined in "${route.url}". available securities: [${Object.keys(securityModules).join(', ')}]`);
                    }
                    for (const securityScope of securityScopes) {
                        if (((_b = securityModules[securityName].validScopes) === null || _b === void 0 ? void 0 : _b.includes(securityScope)) === false &&
                            ((_d = (_c = securityModules[securityName]).validateScope) === null || _d === void 0 ? void 0 : _d.call(_c, securityScope)) ===
                                false) {
                            throw new Error(`[ERROR]: scope "${securityScope}" for security "${securityName}" is invalid "${route.url}".`);
                        }
                    }
                }
            }
        }
    });
    fastify.addHook('preValidation', (request, reply) => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b, _c;
        if (((_a = request.context.schema) === null || _a === void 0 ? void 0 : _a.security) === undefined ||
            ((_b = request.context.schema) === null || _b === void 0 ? void 0 : _b.security.length) === 0) {
            return;
        }
        // all security for route
        const securityGroups = (_c = request.context.schema) === null || _c === void 0 ? void 0 : _c.security;
        // set of security to call to avoid multiple
        const setOfSecurity = new Set(securityGroups.flatMap((s) => Object.keys(s)));
        // set of security solved
        const solvedSecurity = {};
        //solving security
        for (const security of setOfSecurity) {
            const securityData = getSecurityData(securityModules[security].security, request);
            if (securityData !== undefined) {
                try {
                    // @ts-expect-error ts cannot figure out security data to apply
                    solvedSecurity[security] = yield securityModules[security].handle.apply(securityModules[security], Array.isArray(securityData) ? securityData : [securityData]);
                }
                catch (e) {
                    reply.status(500);
                    throw e;
                }
            }
        }
        request.security = {
            passed: [],
            values: solvedSecurity,
        };
        for (const securityGroupIndex in securityGroups) {
            const securityGroup = securityGroups[securityGroupIndex];
            //
            let passed = true;
            for (const [name, scopes] of Object.entries(securityGroup)) {
                if (name in solvedSecurity === false ||
                    solvedSecurity[name] === undefined ||
                    (yield securityModules[name].scopes(solvedSecurity[name], scopes)) === false) {
                    passed = false;
                    break;
                }
            }
            if (passed) {
                request.security.passed.push(Number(securityGroupIndex));
            }
        }
        if (request.security.passed.length === 0) {
            if (Object.values(solvedSecurity).filter((v) => v !== undefined)
                .length === 0) {
                // 401 nessuna autorizzazione fornita
                reply.status(401);
                throw new http_class_1.Unauthorized('no auth method provided', {
                    key: 'error.jwt.missing',
                });
            }
            else {
                // 403 nessuna autorizzazione fornita passa la validazione
                reply.status(403);
                throw new http_class_1.Forbidden('no auth method with grants', {
                    key: 'error.jwt.permission',
                });
            }
        }
    }));
    fastify.addHook('onReady', () => __awaiter(void 0, void 0, void 0, function* () {
        if ('swagger' in fastify === false) {
            throw new Error(`Missing Peer Deps 'fastify-swagger'`);
        }
        const securityDefinitions = Object.entries(securityModules).reduce((acc, [name, sec]) => ({
            [name]: sec.security,
        }), {});
        // @ts-ignore injected by fastify-swagger
        const swagger = fastify.swagger({
            swagger: {
                securityDefinitions,
            },
        });
        let schemePtr = undefined;
        if ('swagger' in swagger) {
            if ('securityDefinitions' in swagger === false) {
                swagger.securityDefinitions = {};
            }
            schemePtr = swagger.securityDefinitions;
        }
        else if ('openapi' in swagger) {
            if ('components' in swagger === false) {
                swagger.components = {};
            }
            if ('securitySchemes' in swagger === false) {
                swagger.components.securitySchemes = {};
            }
            schemePtr = swagger.components.securitySchemes;
        }
        if (schemePtr !== undefined) {
            Object.entries(securityModules).forEach(([name, security]) => {
                schemePtr[name] = security.security;
            });
        }
    }));
    return next();
}), {
    fastify: '3.x',
    name: 'fastify-autosecurity',
    dependencies: ['fastify-swagger'],
});
function loadModule(name, path) {
    const module = require(path);
    if (typeof module === 'function') {
        return module;
    }
    if (typeof module === 'object' &&
        'default' in module &&
        typeof module.default === 'function') {
        return module.default;
    }
    throw new Error(`${ERROR_LABEL}: invalid security module definition (${name}) ${path}. Must export a function`);
}
function getSecurityData(security, request) {
    switch (security.type) {
        case 'http':
            return security.scheme === 'basic'
                ? getBasicAuthSecurityData(security, request)
                : security.scheme === 'bearer'
                    ? getBearerAuthSecurityData(security, request)
                    : invalidSecurity(security);
        case 'basic':
            return getBasicAuthSecurityData(security, request);
        case 'apiKey':
            return getApiKeySecurityData(security, request);
        // case 'oauth2': return getOAuth2SecurityData(security, request)
        // case 'openIdConnect': return getOpenIdConnectSecurityData(security, request)
        default:
            invalidSecurity(security);
    }
}
function invalidSecurity(security) {
    throw new Error(`Invalid security: ${security}`);
}
function getBasicAuthSecurityData(security, request) {
    return request.headers.authorization !== undefined &&
        request.headers.authorization.startsWith('Basic ')
        ? Buffer.from(request.headers.authorization.split(' ')[1], 'base64')
            .toString('ascii')
            .split(':')
        : undefined;
}
function getApiKeySecurityData(security, request) {
    const headerName = security.name.toLocaleLowerCase();
    return request.headers[headerName] !== undefined
        ? [request.headers[headerName]]
        : undefined;
}
function getBearerAuthSecurityData(security, request) {
    return request.headers.authorization !== undefined &&
        request.headers.authorization.startsWith('Bearer ')
        ? request.headers.authorization.split(' ')[1]
        : undefined;
}
__exportStar(require("./types"), exports);
//# sourceMappingURL=index.js.map